<?php

namespace Litepie\Services\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Litepie\Services\Http\Requests\ServiceCategoryRequest;
use Litepie\Services\Interfaces\ServiceCategoryRepositoryInterface;
use Litepie\Services\Models\ServiceCategory;

/**
 * Resource controller class for service_category.
 */
class ServiceCategoryResourceController extends BaseController
{

    /**
     * Initialize service_category resource controller.
     *
     * @param type ServiceCategoryRepositoryInterface $service_category
     *
     * @return null
     */
    public function __construct(ServiceCategoryRepositoryInterface $service_category)
    {
        parent::__construct();
        $this->repository = $service_category;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Litepie\Services\Repositories\Criteria\ServiceCategoryResourceCriteria::class);
    }

    /**
     * Display a list of service_category.
     *
     * @return Response
     */
    public function index(ServiceCategoryRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Litepie\Services\Repositories\Presenter\ServiceCategoryPresenter::class)
                ->$function();
        }

        $service_categories = $this->repository->paginate();

        return $this->response->setMetaTitle(trans('services::service_category.names'))
            ->view('services::service_category.index', true)
            ->data(compact('service_categories', 'view'))
            ->output();
    }

    /**
     * Display service_category.
     *
     * @param Request $request
     * @param Model   $service_category
     *
     * @return Response
     */
    public function show(ServiceCategoryRequest $request, ServiceCategory $service_category)
    {

        if ($service_category->exists) {
            $view = 'services::service_category.show';
        } else {
            $view = 'services::service_category.new';
        }

        return $this->response->setMetaTitle(trans('app.view') . ' ' . trans('services::service_category.name'))
            ->data(compact('service_category'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new service_category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(ServiceCategoryRequest $request)
    {

        $service_category = $this->repository->newInstance([]);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('services::service_category.name')) 
            ->view('services::service_category.create', true) 
            ->data(compact('service_category'))
            ->output();
    }

    /**
     * Create new service_category.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(ServiceCategoryRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $service_category                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('services::service_category.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('services/service_category/' . $service_category->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/services/service_category'))
                ->redirect();
        }

    }

    /**
     * Show service_category for editing.
     *
     * @param Request $request
     * @param Model   $service_category
     *
     * @return Response
     */
    public function edit(ServiceCategoryRequest $request, ServiceCategory $service_category)
    {
        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('services::service_category.name'))
            ->view('services::service_category.edit', true)
            ->data(compact('service_category'))
            ->output();
    }

    /**
     * Update the service_category.
     *
     * @param Request $request
     * @param Model   $service_category
     *
     * @return Response
     */
    public function update(ServiceCategoryRequest $request, ServiceCategory $service_category)
    {
        try {
            $attributes = $request->all();

            $service_category->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('services::service_category.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('services/service_category/' . $service_category->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('services/service_category/' . $service_category->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the service_category.
     *
     * @param Model   $service_category
     *
     * @return Response
     */
    public function destroy(ServiceCategoryRequest $request, ServiceCategory $service_category)
    {
        try {

            $service_category->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('services::service_category.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('services/service_category/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('services/service_category/' . $service_category->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple service_category.
     *
     * @param Model   $service_category
     *
     * @return Response
     */
    public function delete(ServiceCategoryRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('services::service_category.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('services/service_category'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/services/service_category'))
                ->redirect();
        }

    }

    /**
     * Restore deleted service_categories.
     *
     * @param Model   $service_category
     *
     * @return Response
     */
    public function restore(ServiceCategoryRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('services::service_category.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/services/service_category'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/services/service_category/'))
                ->redirect();
        }

    }

}
